// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/UnstableFinalState.hh"
#include "Rivet/Projections/VetoedFinalState.hh"
#include "Rivet/Projections/IdentifiedFinalState.hh"
#include "Rivet/Projections/PromptFinalState.hh"
#include "Rivet/Projections/DressedLeptons.hh"
#include "Rivet/Projections/FastJets.hh"
#include "fastjet/tools/Filter.hh"

namespace Rivet {

  class ATLAS_2015_I1397637 : public Analysis {
  public:

    // Constructor
    ATLAS_2015_I1397637()
      : Analysis("ATLAS_2015_I1397637")
    {
      setNeedsCrossSection(true);
    }



  public:
 
    void init() {
      const FinalState fs(Cuts::abseta < 4.5);

      // Neutrinos for MET
      IdentifiedFinalState nu_id;
      nu_id.acceptNeutrinos();
      PromptFinalState neutrinos(nu_id);
      neutrinos.acceptTauDecays(true);
      addProjection(neutrinos, "neutrinos");

      // Get photons used to dress leptons
      IdentifiedFinalState photons(fs);
      photons.acceptId(PID::PHOTON);

      // Use all bare muons as input to the DressedMuons projection
	    IdentifiedFinalState mu_id(fs);
      mu_id.acceptIdPair(PID::MUON);
      PromptFinalState bare_mu(mu_id);
      bare_mu.acceptTauDecays(true);

      // Use all bare electrons as input to the DressedElectrons projection
	    IdentifiedFinalState el_id(fs);
      el_id.acceptIdPair(PID::ELECTRON);
      PromptFinalState bare_el(el_id);
      bare_el.acceptTauDecays(true);

      // Use all bare leptons including taus for single-lepton filter
      IdentifiedFinalState lep_id(fs);
      lep_id.acceptIdPair(PID::MUON);
      lep_id.acceptIdPair(PID::ELECTRON);
      PromptFinalState bare_lep(lep_id);
      addProjection(bare_lep, "bare_lep");

      UnstableFinalState ufs;
      IdentifiedFinalState tau_id(ufs);
      tau_id.acceptIdPair(PID::TAU);
      PromptFinalState bare_tau(tau_id);
      addProjection(bare_tau, "bare_tau");

	  
      // Muons and electrons must have |eta| < 2.5
	    Cut eta_ranges = Cuts::abseta < 2.5;

      // Get dressed muons and the good muons (pt>25GeV)
      DressedLeptons all_dressed_mu(photons, bare_mu, 0.1, eta_ranges, true, true); 
	    DressedLeptons dressed_mu(photons, bare_mu, 0.1, eta_ranges & (Cuts::pT > 25.0*GeV), true, true); 
	    addProjection(dressed_mu, "muons");


      // Get dressed electrons and the good electrons (pt>25GeV)
      DressedLeptons all_dressed_el(photons, bare_el, 0.1, eta_ranges, true, true); 
  	  DressedLeptons dressed_el(photons, bare_el, 0.1, eta_ranges & (Cuts::pT > 25.0*GeV), true, true); 
	    addProjection(dressed_el, "electrons");	  

	    // Jet clustering
	    VetoedFinalState vfs(fs);
	    vfs.addVetoOnThisFinalState(all_dressed_el);
	    vfs.addVetoOnThisFinalState(all_dressed_mu);
	    vfs.addVetoOnThisFinalState(neutrinos);

	    // small-R jets
	    FastJets jets(vfs, FastJets::ANTIKT, 0.4);
      jets.useInvisibles(JetAlg::ALL_INVISIBLES);
	    jets.useMuons(JetAlg::DECAY_MUONS);
	    addProjection(jets, "jets");
	  
	    // large-R jets
	    FastJets large_jets(vfs, FastJets::ANTIKT, 1.0);
      large_jets.useInvisibles(JetAlg::ALL_INVISIBLES);
	    large_jets.useMuons(JetAlg::DECAY_MUONS);
	    addProjection(large_jets, "fat_jets");

      /// Book histograms:
	    hist = bookHisto1D(1, 1, 1);

    }


    /// Perform the per-event analysis
    void analyze(const Event& event) {
      const double weight = event.weight();
	  
      /*************************
       *  SINGLE LEPTON FILTER  *
       *************************/	  
      // get bare leptons with no cuts
      Particles bare_lep = applyProjection<PromptFinalState>(event, "bare_lep").particles();      
      Particles bare_tau = applyProjection<PromptFinalState>(event, "bare_tau").particles();      
      if (bare_lep.size() + bare_tau.size() != 1)  vetoEvent; // single lepton filter

      /*************************
       *  ELECTRONS AND MUONS  *
       *************************/

	    vector<DressedLepton> electrons = applyProjection<DressedLeptons>(event, "electrons").dressedLeptons();
	    vector<DressedLepton> muons = applyProjection<DressedLeptons>(event, "muons").dressedLeptons();
      if (electrons.size() + muons.size() != 1)  vetoEvent;
      DressedLepton *lepton = NULL;
      if (muons.size())  lepton = &muons[0];
      else               lepton = &electrons[0];

 
      /****************
       *  NEUTRINOS   *
       ****************/
      // Get the neutrinos from the event record (they have pT > 0.0 and |eta| < 4.5 at this stage
      const Particles& neutrinos = applyProjection<PromptFinalState>(event, "neutrinos").particlesByPt();
      FourMomentum met;
      foreach(const Particle& nu, neutrinos)  met += nu.momentum();
      if (met.pT() <= 20*GeV)  vetoEvent;

      /**************
       *    JETS    *
       **************/
	  
	    const Jets& jets  = applyProjection<FastJets>(event, "jets").jets((Cuts::pT>25*GeV) && (Cuts::abseta < 2.5), cmpMomByPt);
	  
  	  /**********************
       *    Jet trimming    *
       **********************/
	    const PseudoJets& fat_jets = applyProjection<FastJets>(event, "fat_jets").pseudoJetsByPt();
	  
	    // trim the jets
		  float Rfilt = 0.3;
		  float ptFrac_min = 0.05; 
		  PseudoJets trimmed_fatJets;
		  fastjet::Filter trimmer(fastjet::JetDefinition(fastjet::kt_algorithm, Rfilt), fastjet::SelectorPtFractionMin(ptFrac_min));
		  foreach (PseudoJet pjet, fat_jets)  trimmed_fatJets += trimmer(pjet);
		  trimmed_fatJets = fastjet::sorted_by_pt(trimmed_fatJets);

		  // use a kt cluster sequence to recluster the trimmed jets so that a d12 can then be obtained from the reclustered jet
		  vector<float> splittingScales;
		  foreach (PseudoJet trimmed_jet, trimmed_fatJets) {
		    PseudoJets tj_constit = trimmed_jet.constituents();
		    fastjet::ClusterSequence kt_cs(tj_constit, fastjet::JetDefinition(fastjet::kt_algorithm, 1.5, fastjet::E_scheme, fastjet::Best));
		    PseudoJets kt_jets = kt_cs.inclusive_jets(); 
		    double d12 = 1.5 * sqrt(kt_jets[0].exclusive_subdmerge(1));
		    splittingScales.push_back(d12);
		  }
				
		  PseudoJets trimmed_jets;
		  for (unsigned int i = 0; i < trimmed_fatJets.size(); ++i) {	
        FourMomentum tj_mom = momentum(trimmed_fatJets[i]);
  	    if (tj_mom.mass()      <= 100*GeV)  continue;
        if (tj_mom.pt()        <= 300*GeV)  continue;
        if (splittingScales[i] <=  40*GeV)  continue;
        if (tj_mom.abseta()    >=     2.0)  continue;
		    trimmed_jets.push_back(trimmed_fatJets[i]);
	  	}
      if (trimmed_jets.empty())  vetoEvent;
		
    
      /*******************
       *    B-TAGGING    *
       *******************/
      Jets bjets, non_bjets;
	    foreach(Jet jet, jets) {
		    if (jet.bTagged())  bjets += jet;
        else                non_bjets += jet;
      }
      if (bjets.empty())  vetoEvent;
	  
	  
      /**************************
       *    BOOSTED SELECTION   *
       **************************/
	   
		  double transmass = _mT(lepton->momentum(), met);
	    if (transmass + met.pt() <= 60*GeV )  vetoEvent;
      bool overlaps = false;
		  int lepJetIndex = -1;
			int index =0;
							
		  foreach (Jet jet, jets) {				
        if (deltaR(jet, lepton->momentum()) < 1.5) {
          overlaps = true;
					lepJetIndex = index; 
				 	break;
		    }
			  ++index;	
		  }
		  if (!overlaps)  vetoEvent;

			int fatJetIndex = -1;
		 	for (unsigned int j = 0; j < trimmed_jets.size(); ++j) {
		   	double dR_fatjet = deltaR(jets[lepJetIndex], momentum(trimmed_jets[j]));
		    double dPhi_fatjet = deltaPhi(lepton->momentum(), momentum(trimmed_jets[j]));
  		  if (dR_fatjet > 1.5 && dPhi_fatjet > 2.3) {
	  			fatJetIndex = j;
		 			break;
				}
	    }
		  if (fatJetIndex < 0)  vetoEvent;

 			bool lepbtag = jets.at(lepJetIndex).bTagged();
	   	bool hadbtag=false;
	 		foreach (Jet bjet, bjets) {
	  		hadbtag |= deltaR(momentum(trimmed_jets.at(fatJetIndex)), bjet.momentum()) < 1.0;
		  }	

		  if (hadbtag || lepbtag) {
		    double hadtop_pt= momentum(trimmed_jets.at(fatJetIndex)).pt();
        hist->fill(hadtop_pt, weight);
			}
    }

    /// Normalise histograms etc., after the run
    void finalize() {
      scale(hist, crossSection() / femtobarn / sumOfWeights());
    }

  private:

    double _mT(const FourMomentum &l, FourMomentum &nu) const {
      return sqrt( 2 * l.pT() * nu.pT() * (1 - cos(deltaPhi(l, nu))) );
    }

    Histo1DPtr hist;

  };

  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(ATLAS_2015_I1397637);

}
